extends Node2D

@onready var game_state = $GameState
@onready var explosion_generator = $ExplosionGenerator
@onready var battle_ui = $CanvasLayer/BattleUI
@onready var player = $HiroPlayer
@onready var ai = $ReedAI
@onready var cutscene_manager = $CutsceneManager
@onready var sound_manager = $SoundManager

func _ready():
	battle_ui.initialize(player.health, ai.health)
	battle_ui.set_player_health(player.health)
	battle_ui.set_enemy_health(ai.health)
	cutscene_manager.step()

func _on_player_defeated():
	if Globals.nsfw == true:
		sound_manager.play_sound("effect_smoke")
		explosion_generator.spawn(player.position)
	var cutscene = load("res://Cutscenes/prototype_defeated.tres")
	cutscene_manager.load_cutscene(cutscene)
	cutscene_manager.step()
		
func _on_cutscene_started(cutscene_name):
	player.can_move = false
	game_state.can_pause = false
	if cutscene_name == "prototype_start":
		ai.set_ai_enabled(false)

func _on_cutscene_finished(cutscene_name: String):
	print("Cutscene " + cutscene_name + " finished")
	player.can_move = true
	game_state.can_pause = true
	if cutscene_name == "prototype_start":
		ai.set_ai_enabled(true)


func _on_cutscene_manager_issued_command(command: CutsceneCommand, manager: CutsceneManager):
	if command is TurnCharacterCommand or command is LookAtTargetCommand:
		var character: CharacterBase = manager.get_node(command.character)
		var face_direction: Vector2 = command.get_direction(manager)
		assert(character is CharacterBase)
		character.set_animation_direction(face_direction)
		manager.step()
